alias linkedlist_print {
  var %name = $1
  if ($linkedlist(%name)) {
    var %node = $linkedlist_node(%name,0).next
    while (%node != $null) {
      ;%node is the reference for our current node

      echo -a %node $+ : $linkedlist_node(%name,%node).item - $linkedlist_node(%name,%node).data

      %node = $linkedlist_node(%name,%node).next
    }
  }
}

alias linkedlist_print2 {
  var %name = $1
  if ($linkedlist(%name)) {
    var %node.current = 0
    while (%node.current != $null) {
      var %next = $linkedlist_node(%name,%node.current).next
      ;%next is a reference to the next node (may be null.. so make sure you check)

      if (%next) {
        ;this is almost identical to the method used in linkedlist_print, however this can be more
        ;useful in some cases, where you plan to remove elements (see more examples further below)
        ;with the linkedlist_print method, you would have to use a variable to store the previous node
        echo -a %next $+ : $linkedlist_node(%name,%next).item - $linkedlist_node(%name,%next).data
      }

      %node.current = %next
    }
  }
}

alias linkedlist_removeexample {
  ;this is an example of dynamically removing data from the list whilst still continuing to iterate through it
  ;something that can be very handy.. this example removes all non numeric data from our linked list
  var %llname = tests
  linkedlist_make %llname
  linkedlist_insert %llname foo $
  linkedlist_insert %llname oof 4
  linkedlist_insert %llname bar 7
  linkedlist_insert %llname baz 9
  linkedlist_insert %llname bob x
  linkedlist_insert %llname jay pp
  linkedlist_insert %llname jay 5g535
  linkedlist_insert %llname joe 17
  linkedlist_insert %llname rob -
  linkedlist_print %llname
  echo -a -------- removing --------
  var %node.current = 0
  while (%node.current != $null) {
    var %next = $linkedlist_node(%llname,%node.current).next
    var %next.data = $linkedlist_node(%llname,%next).data
    ;we have to access the _next_ element only. We can only remove it if we have the element before it
    if ((%next) && (%next.data !isnum)) {
      linkedlist_removeAfter %llname %node.current
    }
    else {
      %node.current = %next
    }
  }
  linkedlist_print2 %llname
  linkedlist_free %llname
}

alias linkedlist_removeexample2 {
  var %llname = tests
  linkedlist_free %llname
  linkedlist_make %llname
  linkedlist_insert %llname foo $
  linkedlist_insert %llname oof 4
  linkedlist_insert %llname bar 7
  linkedlist_insert %llname baz 9
  linkedlist_insert %llname bob x
  linkedlist_insert %llname jay pp
  linkedlist_insert %llname jay 5g535
  linkedlist_insert %llname joe 17
  linkedlist_insert %llname rob -
  linkedlist_print %llname
  echo -a ----removed:----
  var %prev = 0
  var %node = $linkedlist_node(%llname,%prev).next
  while (%node != $null) {
    var %node.data = $linkedlist_node(%llname,%node).data
    if (%node.data !isnum) {
      linkedlist_removeAfter %llname %prev
      ;removeAfter the PREVIOUS node
      ;note: the current node becomes defunct after this, since its been removed
    }
    else {
      %prev = %node
    }
    %node = $linkedlist_node(%llname,%prev).next
  }
  linkedlist_print2 %llname
}

alias linkedlist_sortexample {
  linkedlist_free test
  linkedlist_make test
  linkedlist_insert test num 6
  linkedlist_insert test num 3
  linkedlist_insert test num 8
  linkedlist_insert test num a
  linkedlist_insert test num 2
  linkedlist_insert test num 18
  linkedlist_insert test num A
  linkedlist_insert test num BB
  linkedlist_insert test num 2
  linkedlist_insert test num 17
  linkedlist_insert test num 1
  linkedlist_insert test num BC
  linkedlist_print test
  echo -s ---sorted:---
  linkedlist_sort -s test asc 
  linkedlist_print test
  echo -s --------
}

alias linkedlist_sortexample2 {
  linkedlist_free test
  linkedlist_make test
  linkedlist_insert test my 1
  linkedlist_insert test long 1
  linkedlist_insert test story 1
  linkedlist_insert test about 1
  linkedlist_insert test nothing 1
  linkedlist_insert test really 1
  linkedlist_insert test but 1
  linkedlist_insert test well 1
  linkedlist_insert test ok 1
  linkedlist_insert test major 1
  linkedlist_insert test lulz 1
  linkedlist_insert test 2u 1
  linkedlist_print test
  echo -s ---sorted:---
  linkedlist_sort -s test item desc 
  linkedlist_print test
  echo -s --------
}
